import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:okane/database/collections/account.dart';
import 'package:okane/database/database.dart';
import 'package:okane/i18n/strings.g.dart';
import 'package:okane/ui/state/core.dart';
import 'package:okane/ui/utils.dart';
import 'package:okane/ui/widgets/piechart_card.dart';

class TotalBalanceCard extends StatelessWidget {
  const TotalBalanceCard({super.key});

  Future<double> _getTotalBalance(List<Account> accounts) async {
    if (accounts.isEmpty) {
      return 0;
    }

    final results = await Future.wait(accounts.map(getTotalBalance).toList());
    final loanSum = await getTotalLoanSum();

    return results.reduce((acc, val) => acc + val) + loanSum;
  }

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<CoreCubit, CoreState>(
      builder: (context, state) {
        return ResponsiveCard(
          titleText: t.pages.accounts.totalBalance.title,
          child: Padding(
            padding: EdgeInsets.all(16),
            child: FutureBuilder(
              future: _getTotalBalance(state.accounts),
              builder: (context, snapshot) {
                return Text(
                  snapshot.hasData
                      ? formatCurrency(snapshot.data!)
                      : t.pages.accounts.totalBalance.loading,
                  style: Theme.of(context).textTheme.bodyLarge,
                );
              },
            ),
          ),
        );
      },
    );
  }
}
